//Моделируем динамику
function [SystemOutputChatacteristics,..//Выходные характеристики системы
          SystemProps..//Свойства системы
          ]=ComputingExperiment(SystemOutputChatacteristicsNames,...//Имена выходных характеристик системы
                                SystemShema,..//Cхема системы или имя файла схемы ситемы
                                SystemParametrsNames,...//Имена параметров
                                SystemParametrsValues,...//Значения параметров
                                absolutTolErr,..//Максимальное значение абсолютной ошибки
                                relativeTolErr,..//Минимальное значение относительной ошибки
                                simulationTime,..//Время симуляции
                                timeStepMax,..//Максимальный шаг интергирования по времени
                                Solver,..//Решатель
                                toleranceTime..//Максимальная ошибка времени
                                )//Моделирование возможных динамик
    //Проверяем число аргументов
    nArgs = argn(2);
    if nArgs < 2 then
        error('Function ComputingExperiment takes 2 or more arguments (maximum 10 arguments)');
    end
    
    //Проверяем аргументы
    select nArgs
    case 2 then
        SystemParametrsNames = [];
        SystemParametrsValues = [];
        absolutTolErr = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 3 then
        if isempty(SystemParametrsNames) then
            SystemParametrsValues = [];
            absolutTolErr = [];
            relativeTolErr = [];
            simulationTime = [];
            timeStepMax = [];
            Solver = [];
            toleranceTime = [];
        else
            if type(SystemParametrsNames) == 1 then
                absolutTolErr = SystemParametrsNames;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument absolutTolErr of function ComputingExperiment must be matrix of real');
            end
        end
    case 4 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                absolutTolErr = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                absolutTolErr = [];
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 5 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 6 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 7 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                toleranceTime = [];
            case 10 then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 8 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                Solver = [];
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                toleranceTime = timeStepMax;
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
            case 10 then
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 9 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                toleranceTime = [];
            else
                error('System parametrs names and values must be empty together or not empty together');
            end
        else
            if type(SystemParametrsNames) == 10 then
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames must be matrix of string');
            end
        end
    end
    
    //Проверяем корректность типов данных аргументов
    if (~isempty(SystemOutputChatacteristicsNames)) &&..
       (type(SystemOutputChatacteristicsNames) ~= 10) then
        error('The SystemOutputChatacteristicsNames argument of ComputingExperiment function is array of string');
    end
    
    //Задаем параметры и контекст моделирования
    SystemShema = SetDiagramParameters(SystemShema,...//Схема системы
                                       SystemParametrsNames,...//Имена параметров
                                       SystemParametrsValues,...//Значения параметров
                                       absolutTolErr,..//Максимальное значение абсолютной ошибки
                                       relativeTolErr,..//Минимальное значение относительной ошибки
                                       simulationTime,..//Время симуляции
                                       timeStepMax,..//Максимальный шаг интергирования по времени
                                       Solver,..//Решатель
                                       toleranceTime..//Максимальная ошибка времени
                                       )
              
    //Приводим имена выходных характеристик системы к виду матрица-строка
    SystemOutputChatacteristicsNames = matrix(SystemOutputChatacteristicsNames, 1, -1);
                            
    //Запускаем моделирование динамики
    xcos_simulate(SystemShema, 4);
    
    //Получаем значения выходных характеристик системы
    SystemOutputChatacteristics = evstr(SystemOutputChatacteristicsNames);
    
    //Возвращаем свойства системы
    SystemProps = SystemShema.props;
endfunction
