//Функция генерации обучающих данных (для динамики выходных характеристик)
function [SystemOutputChatacteristics..//Выходные характеристики системы
          ]=ModelingSystemDynamicEvidence(SystemOutputCharacteristicsNames,..//Имена выходных характеристик системы
                                          SystemOutputCharacteristicsIndexes,..//Индексы выходных характеристик системы (в именах)
                                          SystemShema,..//Cхема системы или имя файла схемы ситемы
                                          SystemOutputCharacteristicsCompressionFunctions,..//Функция сжатия динамики
                                          SystemParametrsNames,..//Имена параметров, определяющих динамику
                                          SystemParametrsValues,..//Значения параметров, определяющих динамику
                                          SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                          SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                          absolutTolErr,..//Максимальное значение абсолютной ошибки
                                          relativeTolErr,..//Минимальное значение относительной ошибки
                                          simulationTime,..//Время симуляции
                                          timeStepMax,..//Максимальный шаг интергирования по времени
                                          Solver,..//Решатель
                                          toleranceTime..//Максимальная ошибка времени
                                          )
    //Проверяем число аргументов
    nArg = argn(2);//Число аргументов
    if nArg < 6 then
        error('Function ModelingSystemDynamicEvidence takes 6 or more arguments');
    end
    
    //Восполняем недостающие аргументы
    select nArg
    case 6 then
        SystemHyperParametrsNames = [];
        SystemHyperParametrsValues = [];
        absolutTolErr = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 7 then
        absolutTolErr = SystemHyperParametrsNames;
        SystemHyperParametrsNames = [];
        SystemHyperParametrsValues = [];
        relativeTolErr = [];
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 8 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            absolutTolErr = [];
            relativeTolErr = [];
        else
            absolutTolErr = SystemHyperParametrsNames;
            relativeTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        simulationTime = [];
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 9 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            relativeTolErr = [];
            simulationTime = [];
        else
            simulationTime = absolutTolErr;
            absolutTolErr = SystemHyperParametrsNames;
            relativeTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        timeStepMax = [];
        Solver = [];
        toleranceTime = [];
    case 10 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            simulationTime = [];
            timeStepMax = [];
        else
            timeStepMax = relativeTolErr;
            simulationTime = absolutTolErr;
            absolutTolErr = SystemHyperParametrsNames;
            relativeTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        Solver = [];
        toleranceTime = [];
    case 11 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            timeStepMax = [];
            Solver = [];
        else
            Solver = simulationTime;
            timeStepMax = relativeTolErr;
            simulationTime = absolutTolErr;
            absolutTolErr = SystemHyperParametrsNames;
            relativeTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        toleranceTime = [];
    case 12 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            Solver = [];
            toleranceTime = [];
        else
            toleranceTime = timeStepMax;
            Solver = simulationTime;
            timeStepMax = relativeTolErr;
            simulationTime = absolutTolErr;
            absolutTolErr = SystemHyperParametrsNames;
            relativeTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
    case 13 then
        toleranceTime = [];
    end
           
    //Приволим аргументы к виду матрица - строка
    SystemOutputCharacteristicsIndexes = matrix(SystemOutputCharacteristicsIndexes, 1, -1);
    SystemOutputCharacteristicsCompressionFunctions = matrix(SystemOutputCharacteristicsCompressionFunctions, 1, -1);
        
    //Проверяем корректность аргументов
    if (~isempty(SystemOutputCharacteristicsNames)) &&..
       (type(SystemOutputCharacteristicsNames) ~= 10) then
        error('The SystemOutputCharacteristicsNames argument of function ModelingSystemDynamicEvidence is matrix of strings');
    end
    if (~isempty(SystemOutputCharacteristicsIndexes)) &&..
       (type(SystemOutputCharacteristicsIndexes) ~= 1) then
        error('The SystemOutputCharacteristicsIndexes argument of function ModelingSystemDynamicEvidence is matrix of integer');
    end
    sizeSystemOutputCharacteristicsNames = size(SystemOutputCharacteristicsNames,'*');
    if sizeSystemOutputCharacteristicsNames ~= size(SystemOutputCharacteristicsIndexes,'*') then
        error('The sizes of arguments SystemOutputCharacteristicsNames and SystemOutputCharacteristicsIndexes of function ModelingSystemDynamicEvidence must be equality');
    end
    if type(SystemOutputCharacteristicsCompressionFunctions) == 10 then
        //Преобразовываем имена функций сжатия в cell-массив функций сжатия
        SystemOutputCharacteristicsCompressionFunctions = CreateSystemOutputCharacteristicsCompressionFunctions(SystemOutputCharacteristicsCompressionFunctions);
    end
    if type(SystemOutputCharacteristicsCompressionFunctions) == 17 then
        for ind=1:size(SystemOutputCharacteristicsCompressionFunctions,'*')
            if type(SystemOutputCharacteristicsCompressionFunctions{ind}) ~= 13 then
                error('The SystemOutputCharacteristicsCompressionFunctions argument of function ModelingSystemDynamicEvidence is cell-array of functions');
            end
        end
    else
        error('The SystemOutputCharacteristicsCompressionFunctions argument of function ModelingSystemDynamicEvidence is cell-array of functions or string array of function''s names');
    end
    if size(SystemOutputCharacteristicsCompressionFunctions,'*') ~= sizeSystemOutputCharacteristicsNames then
        error('The number of compression functions must be equal to the number of dynamic curves (dimnesion of argument SystemOutputCharacteristicsNames)');
    end
    if (~isempty(SystemParametrsNames)) &&..
       (type(SystemParametrsNames) ~= 10) then
        error('The SystemParametrsNames argument of function ModelingSystemDynamicExtInfEvidence is matrix of strings');
    end
    if (~isempty(SystemParametrsValues)) &&..
       (type(SystemParametrsValues) ~= 10) &&..
       (type(SystemParametrsValues) ~= 1) &&..
       (type(SystemParametrsValues) ~= 4) then
        error('The SystemParametrsValues argument of function ModelingSystemDynamicExtInfEvidence is matrix of strings, real, booleans');
    end
    if size(SystemParametrsNames,'*') ~= size(SystemParametrsValues)(2) then
        error('The number of parametrs''s names and number of values of this parametrs must be equality');
    end
    
    //Получаем список уже открытых графических окон
    Wins_begin = winsid();
      
    //Загружаем схему системы и задаем гиперпараметры
    SystemShema = SetDiagramParameters(SystemShema,..
                                       SystemHyperParametrsNames,..
                                       SystemHyperParametrsValues);
    
    //Векторизуемая функция вычислительного эксперимента
    nCurves = size(SystemParametrsValues)(1);//Число возможных динамик
    SystemOutputChatacteristics = [];
    for ind=1:nCurves
        //Проводим вычислительный эксперимент
        Dynamics = ComputingExperiment(SystemOutputCharacteristicsNames,...//Имена выходных характеристик системы
                                       SystemShema,...//Схема системы
                                       SystemParametrsNames,...//Имена параметров
                                       SystemParametrsValues(ind,:),...//Значения параметров
                                       absolutTolErr(ind),..//Максимальное значение абсолютной ошибки
                                       relativeTolErr(ind),..//Минимальное значение относительной ошибки
                                       simulationTime(ind),..//Время симуляции
                                       timeStepMax(ind),..//Максимальный шаг интергирования по времени
                                       Solver(ind),..//Решатель
                                       toleranceTime(ind)..//Максимальная ошибка времени
                                       );
        
        //Сжимаем полученные динамики
        SystemOutputChatacteristics = [SystemOutputChatacteristics;..
                                       CountDynamicsEvidences(Dynamics,...//Массив динамик
                                                              SystemOutputCharacteristicsIndexes,...//Индексы динамик (в именах)
                                                              SystemOutputCharacteristicsCompressionFunctions...//Функция фильтра динамики
                                                              )]; 
    end
    
    //Закрываем графические окна, открывшиеся в результате вычислительного эксперимента
    Wins_end = winsid();//Все открытые графические окна
    Wins = setdiff(Wins_end,Wins_begin);//Получаем графические окна, которые были открыты, и которые надо закрыть
    for win=matrix(Wins,1,-1)//Закрываем открытые графические окна
        delete(scf(win));//Закрываем окно
    end
endfunction

//Расчет проекции полученной динамики
function [Evidences...//Параметры
          ]=CountDynamicsEvidences(Dynamics,...//Массив динамик
                                   DynamicsIndexes,...//Индексы динамик (в именах)
                                   DynamicCompressionFunctions...//Функция фильтра динамики
                                   )
    //Проверяем корректность аргументов
    if isempty(Dynamics) || isempty(DynamicsIndexes) then
        Evidences = [];
    end
    
    //Выполняем фильтрацию
    nDynamics = size(DynamicsIndexes,'*');//Число сжимаемых динамик
    Evidences = [];
    for ind=1:nDynamics//Вспомогательная функция фильтрации по индексу
        parametrs = DynamicCompressionFunctions{ind}(Dynamics(ind).time,...
                                                     Dynamics(ind).values(:,DynamicsIndexes(ind)));
        Evidences = [Evidences, parametrs];
    end
endfunction

//Создание cell-массива функций из имен функций
function [SystemOutputCharacteristicsCompressionFunctions]=CreateSystemOutputCharacteristicsCompressionFunctions(SystemOutputCharacteristicsCompressionFunctionsNames)
    //Формируем строки, выполнив которые получим массив функций сжатия
    nSystemOutputCharacteristicsCompressionFunctions = size(SystemOutputCharacteristicsCompressionFunctionsNames,'*');//Число функций сжатия
    SystemOutputCharacteristicsCompressionFunctions = cell(1,nSystemOutputCharacteristicsCompressionFunctions);//Сеll-массив функций сжатия
    indSystemOutputCharacteristicsCompressionFunctions = [1:nSystemOutputCharacteristicsCompressionFunctions];//Индексы функций сжатия
    strSystemOutputCharacteristicsCompressionFunctions = 'SystemOutputCharacteristicsCompressionFunctions{' + string(indSystemOutputCharacteristicsCompressionFunctions) + '}=' + SystemOutputCharacteristicsCompressionFunctionsNames;//Строка формирования массива функций сжатия
    
    //Выполнение строк
    execstr(strSystemOutputCharacteristicsCompressionFunctions);
endfunction
