//Моделирование динамики на основе идентификации по начальному участку
function [SystemOutputChatacteristics,..//Выходные характеристики системы
          SystemIdentificationParametersValues,..//Значения идентифицированных параметров системы
          SystemModesParametersIndexes,..//Индексы параметров системы, определяющих ее режимы
          SystemModesParametersValues,..//Значения параметров системы, определяющих ее режимы
          TimesMomemts..//Моменты времени
          ]=ModelingSystemIdentificationDynamic(SystemIdentificateShema,..//Идентификационная схема системы или имя файла идентификационной схемы ситемы
                                                SystemIdentificateOutputCharacteristicsNames,..//Имена выходных характеристик системы, по которым осуществляется идентификация (по отклонениям этих величин от соответсвующих экспериментальных значений)
                                                SystemIdentificateOutputCharacteristicsIndexes,..//Индексы выходных характеристик системы (в именах), по которым осуществляется идентификация (по отклонениям этих величин от соответсвующих экспериментальных значений)
                                                SystemIdentificateOutputCharacteristicsCompressionFunctions,..//Функция сжатия динамик выходных характеристик системы, по которым осуществляется идентификация (по отклонениям этих величин от соответсвующих экспериментальных значений)
                                                SystemIdentificateModesParametersNames,..//Имена параметров системы, определяющих ее контрольные режимы
                                                SystemIdentificateModesParametersValues,..//Значения параметров системы, определяющих ее контрольные режимы
                                                SystemIdentificationParametersNames,..//Имена идентифицируемых параметров системы
                                                SystemIdentificationParametersMinValues,..//Миниальные значения идентифицируемых параметров системы
                                                SystemIdentificationParametersMaxValues,..//Максимальные значения идентифицируемых параметров системы
                                                IdentificationFunction,..//Идентификационная функция
                                                SystemShema,..//Cхема системы или имя файла схемы ситемы
                                                SystemOutputCharacteristicsNames,..//Имена выходных характеристик системы
                                                SystemOutputCharacteristicsIndexes,..//Индексы выходных характеристик системы (в именах)
                                                SystemOutputCharacteristicsProcessingType,..//Тип обработки выходных характеристик системы
                                                SystemOutputCharacteristicsProcessingAttributes,..//Аттрибуты обработки выходных характеристик системы (функции сжатия динамики или временные сетки)
                                                SystemModesParametersNames,..//Имена параметров системы, определяющих ее режимы
                                                SystemModesParametersValues,..//Значения параметров системы, определяющих ее режимы
                                                SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                                SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                                identificationAbsolutTolErr,..//Максимальное значение абсолютной ошибки для режимов идентификации системы
                                                absolutTolErr,..//Максимальное значение абсолютной ошибки
                                                identificationRelativeTolErr,..//Минимальное значение относительной ошибки для режимов идентификации системы
                                                relativeTolErr,..//Минимальное значение относительной ошибки
                                                identificationSimulationTime,..//Время симуляции для режимов идентификации системы
                                                simulationTime,..//Время симуляции
                                                identificationTimeStepMax,..//Максимальный шаг интергирования по времени для режимов идентификации системы
                                                timeStepMax,..//Максимальный шаг интергирования по времени
                                                identificationSolver,..//Решатель для режимов идентификации системы
                                                Solver,..//Решатель
                                                identificationToleranceTime,..//Максимальная ошибка времени для режимов идентификации системы
                                                toleranceTime..//Максимальная ошибка времени
                                                )
    //Проверяем число аргументов
    [nRez,nArg] = argn();//Число аргументов
    if nArg < 17 then
        error('Function ModelingSystemIdentificationDynamic takes 17 or more arguments');
    end
    
    //Восполняем недостающие аргументы
    select nArg
    case 17 then
        SystemHyperParametrsNames = [];
        SystemHyperParametrsValues = [];
        identificationAbsolutTolErr = [];
        absolutTolErr = [];
        identificationRelativeTolErr = [];
        relativeTolErr = [];
        identificationSimulationTime = [];
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 18 then
        identificationAbsolutTolErr = SystemHyperParametrsNames;
        SystemHyperParametrsNames = [];
        SystemHyperParametrsValues = [];
        absolutTolErr = [];
        identificationRelativeTolErr = [];
        relativeTolErr = [];
        identificationSimulationTime = [];
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 19 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationAbsolutTolErr = [];
            absolutTolErr = [];
        else
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        identificationRelativeTolErr = [];
        relativeTolErr = [];
        identificationSimulationTime = [];
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 20 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            absolutTolErr = [];
            identificationRelativeTolErr = [];
        else
            identificationRelativeTolErr = identificationAbsolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        relativeTolErr = [];
        identificationSimulationTime = [];
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 21 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationRelativeTolErr = [];
            relativeTolErr = [];
        else
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        identificationSimulationTime = [];
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 22 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            relativeTolErr = [];
            identificationSimulationTime = [];
        else
            identificationSimulationTime = identificationRelativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        simulationTime = [];
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 23 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationSimulationTime = [];
            simulationTime = [];
        else
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        identificationTimeStepMax = [];
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 24 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            simulationTime = [];
            identificationTimeStepMax = [];
        else
            identificationTimeStepMax = identificationSimulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        timeStepMax = [];
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 25 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationTimeStepMax = [];
            timeStepMax = [];
        else
            identificationTimeStepMax = identificationSimulationTime;
            timeStepMax = simulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        identificationSolver = [];
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 26 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            timeStepMax = [];
            identificationSolver = [];
        else
            identificationSolver = identificationTimeStepMax;
            identificationTimeStepMax = identificationSimulationTime;
            timeStepMax = simulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        Solver = [];
        identificationToleranceTime = [];
        toleranceTime = [];
    case 27 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationSolver = [];
            Solver = [];
        else
            identificationSolver = identificationTimeStepMax;
            Solver = timeStepMax;
            identificationTimeStepMax = identificationSimulationTime;
            timeStepMax = simulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        identificationToleranceTime = [];
        toleranceTime = [];
    case 28 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            Solver = [];
            identificationToleranceTime = [];
        else
            identificationToleranceTime = identificationSolver;
            identificationSolver = identificationTimeStepMax;
            Solver = timeStepMax;
            identificationTimeStepMax = identificationSimulationTime;
            timeStepMax = simulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
        toleranceTime = [];
    case 29 then
        if (type(SystemHyperParametrsNames) == 10) ||..
           isempty(SystemHyperParametrsNames) then
            identificationToleranceTime = [];
            toleranceTime = [];
        else
            identificationToleranceTime = identificationSolver;
            toleranceTime = Solver;
            identificationSolver = identificationTimeStepMax;
            Solver = timeStepMax;
            identificationTimeStepMax = identificationSimulationTime;
            timeStepMax = simulationTime;
            identificationSimulationTime = identificationRelativeTolErr;
            simulationTime = relativeTolErr;
            identificationRelativeTolErr = identificationAbsolutTolErr;
            relativeTolErr = absolutTolErr;
            identificationAbsolutTolErr = SystemHyperParametrsNames;
            absolutTolErr = SystemHyperParametrsValues;
            SystemHyperParametrsNames = [];
            SystemHyperParametrsValues = [];
        end
    case 30 then
        toleranceTime = [];
    end
    
    //Проверяем корректность аргументов
    typeIdentificationFunction = type(IdentificationFunction);//Тип идентификационной функции
    if typeIdentificationFunction == 10 then
        execstr('IdentificationFunction = ' + IdentificationFunction);
    elseif typeIdentificationFunction ~= 13 then
        error('The argument IdentificationFunction of function ModelingSystemIdentificationDynamic must be function type or string type (name of function)');
    end
    if (type(SystemIdentificationParametersNames) ~= 10) ||..
       (size(SystemIdentificationParametersNames)(1) > 1) ||..
       isempty(SystemIdentificationParametersNames) then
        error('The argument SystemIdentificationParametersNames of function ModelingSystemIdentificationDynamic must be matrix of string 1xM');
    end
    if (type(SystemIdentificationParametersMinValues) ~= 1) &&..
       (type(SystemIdentificationParametersMinValues) ~= 4) &&..
       (type(SystemIdentificationParametersMinValues) ~= 10) then
        error('The argument SystemIdentificationParametersMinValues of function ModelingSystemIdentificationDynamic must be matrix of real, boolean or string NxM');
    end
    if (type(SystemIdentificationParametersMaxValues) ~= 1) &&..
       (type(SystemIdentificationParametersMaxValues) ~= 4) &&..
       (type(SystemIdentificationParametersMaxValues) ~= 10) then
        error('The argument SystemIdentificationParametersMaxValues of function ModelingSystemIdentificationDynamic must be matrix of real, boolean or string NxM');
    end
    if (size(SystemIdentificationParametersMinValues)(2) ~= size(SystemIdentificationParametersNames)(2)) ||..
       (size(SystemIdentificationParametersMaxValues)(2) ~= size(SystemIdentificationParametersNames)(2)) ||..
       (size(SystemIdentificationParametersMaxValues)(1) ~= size(SystemIdentificationParametersMinValues)(1)) then
        error('The argument SystemIdentificationParametersNames of function ModelingSystemIdentificationDynamic must be matrix of string 1xM and arguments SystemIdentificationParametersMinValues and SystemIdentificationParametersMaxValues of function ModelingSystemIdentificationDynamic must be matrix of real, boolean or string NxM');
    end
    
    //Выполняем идентификацию параметров
    SystemIdentificateShema = SetDiagramParameters(SystemIdentificateShema,..
                                                   SystemHyperParametrsNames,..
                                                   SystemHyperParametrsValues);//Задаем гиперпараметры для идентификационной схемы системы
    function [OutputValues]=fOutputValues(parameters)
        OutputValues = [];//Выходные характеристики
        for curParameters=parameters'
            OutputValues = cat(3,OutputValues,..
                               ModelingSystemDynamicEvidence(SystemIdentificateOutputCharacteristicsNames,..
                                                             SystemIdentificateOutputCharacteristicsIndexes,..
                                                             SystemIdentificateShema,..
                                                             SystemIdentificateOutputCharacteristicsCompressionFunctions,..
                                                             SystemIdentificateModesParametersNames,..
                                                             SystemIdentificateModesParametersValues,..
                                                             SystemIdentificationParametersNames,..
                                                             curParameters',..
                                                             identificationAbsolutTolErr,..
                                                             identificationRelativeTolErr,..//Минимальное значение относительной ошибки
                                                             identificationSimulationTime,..//Время симуляции
                                                             identificationTimeStepMax,..//Максимальный шаг интергирования по времени
                                                             identificationSolver,..//Решатель
                                                             identificationToleranceTime..//Максимальная ошибка времени
                                                             ));
        end
    endfunction
    SystemIdentificationParametersValues = IdentificationFunction(fOutputValues,..
                                                                  SystemIdentificationParametersMinValues,..
                                                                  SystemIdentificationParametersMaxValues);
    
    //Рассчитываем динамику в основном режиме
    SystemShema = SetDiagramParameters(SystemShema,..
                                       SystemHyperParametrsNames,..
                                       SystemHyperParametrsValues);//Задаем гиперпараметры для схемы системы
    select SystemOutputCharacteristicsProcessingType
    case 'Evidence' then
        //Моделируем динамику для идентифицированных параметров
        SystemOutputChatacteristics = ModelingSystemDynamicEvidence(SystemOutputCharacteristicsNames,..
                                                                    SystemOutputCharacteristicsIndexes,..
                                                                    SystemShema,..
                                                                    SystemOutputCharacteristicsProcessingAttributes,..
                                                                    SystemModesParametersNames,..
                                                                    SystemModesParametersValues,..
                                                                    SystemIdentificationParametersNames,..
                                                                    SystemIdentificationParametersValues,..
                                                                    absolutTolErr,relativeTolErr,..
                                                                    simulationTime,timeStepMax,..
                                                                    Solver,toleranceTime);
        
        //Заполняем прочие выходные параметры функции
        if nRez > 2 then
            SystemModesParametersIndexes = [];
        end
        if nRez > 4 then
            TimesMomemts = [];
        end
    case 'TimeGrid' then
        //Моделируем динамику для идентифицированных параметров
        select nRez
        case 1 then
            [SystemOutputChatacteristics..//Выходные характеристики
             ]=ModelingSystemDynamicTimeGrid(SystemOutputCharacteristicsNames,..
                                             SystemOutputCharacteristicsIndexes,..
                                             SystemShema,..
                                             SystemOutputCharacteristicsProcessingAttributes,..
                                             SystemModesParametersNames,..
                                             SystemModesParametersValues,..
                                             SystemIdentificationParametersNames,..
                                             SystemIdentificationParametersValues,..
                                             absolutTolErr,relativeTolErr,..
                                             simulationTime,timeStepMax,..
                                             Solver,toleranceTime);
        case 2 then
            [SystemOutputChatacteristics..//Выходные характеристики
             ]=ModelingSystemDynamicTimeGrid(SystemOutputCharacteristicsNames,..
                                             SystemOutputCharacteristicsIndexes,..
                                             SystemShema,..
                                             SystemOutputCharacteristicsProcessingAttributes,..
                                             SystemModesParametersNames,..
                                             SystemModesParametersValues,..
                                             SystemIdentificationParametersNames,..
                                             SystemIdentificationParametersValues,..
                                             absolutTolErr,relativeTolErr,..
                                             simulationTime,timeStepMax,..
                                             Solver,toleranceTime);
        case 3 then
            [SystemOutputChatacteristics,..//Выходные характеристики
             SystemModesParametersIndexes..//Индексы параметров системы, для которых рассчитывались выходные характеристики
             ]=ModelingSystemDynamicTimeGrid(SystemOutputCharacteristicsNames,..
                                             SystemOutputCharacteristicsIndexes,..
                                             SystemShema,..
                                             SystemOutputCharacteristicsProcessingAttributes,..
                                             SystemModesParametersNames,..
                                             SystemModesParametersValues,..
                                             SystemIdentificationParametersNames,..
                                             SystemIdentificationParametersValues,..
                                             absolutTolErr,relativeTolErr,..
                                             simulationTime,timeStepMax,..
                                             Solver,toleranceTime);
        case 4 then
            [SystemOutputChatacteristics,..//Выходные характеристики
             SystemModesParametersIndexes,..//Индексы параметров системы, для которых рассчитывались выходные характеристики
             SystemModesParametersValues..//Параметры системы, для которых рассчитывались выходные характеристики
             ]=ModelingSystemDynamicTimeGrid(SystemOutputCharacteristicsNames,..
                                             SystemOutputCharacteristicsIndexes,..
                                             SystemShema,..
                                             SystemOutputCharacteristicsProcessingAttributes,..
                                             SystemModesParametersNames,..
                                             SystemModesParametersValues,..
                                             SystemIdentificationParametersNames,..
                                             SystemIdentificationParametersValues,..
                                             absolutTolErr,relativeTolErr,..
                                             simulationTime,timeStepMax,..
                                             Solver,toleranceTime);
        case 5 then
            [SystemOutputChatacteristics,..//Выходные характеристики
             SystemModesParametersIndexes,..//Индексы параметров системы, для которых рассчитывались выходные характеристики
             SystemModesParametersValues,..//Параметры системы, для которых рассчитывались выходные характеристики
             TimesMomemts..//Моменты времени
             ]=ModelingSystemDynamicTimeGrid(SystemOutputCharacteristicsNames,..
                                             SystemOutputCharacteristicsIndexes,..
                                             SystemShema,..
                                             SystemOutputCharacteristicsProcessingAttributes,..
                                             SystemModesParametersNames,..
                                             SystemModesParametersValues,..
                                             SystemIdentificationParametersNames,..
                                             SystemIdentificationParametersValues,..
                                             absolutTolErr,relativeTolErr,..
                                             simulationTime,timeStepMax,..
                                             Solver,toleranceTime);
        end
    else
        error('The argument SystemOutputCharacteristicsProcessingType of function ModelingSystemIdentificationDynamic must be eguality ''Evidence'' or ''TimeGrid''');
    end
endfunction
