//Задаем параметры диаграммы
function [SystemShema]=SetDiagramParameters(SystemShema,...//Схема системы или имя файла схемы ситемы
                                            SystemParametrsNames,...//Имена параметров
                                            SystemParametrsValues,...//Значения параметров
                                            absolutTolErr,..//Максимальное значение абсолютной ошибки
                                            relativeTolErr,..//Минимальное значение относительной ошибки
                                            simulationTime,..//Время симуляции
                                            timeStepMax,..//Максимальный шаг интергирования по времени
                                            Solver,..//Решатель
                                            toleranceTime..//Максимальная ошибка времени
                                            )
    //Проверяем число аргументов
    nArgs = argn(2);
    if nArgs < 1 then
        error('Function SetDiagramParameters takes 1 or more arguments');
    end
    
    //Проверяем корректность аргумента имени схемы
    typeSystemShema = type(SystemShema);//Тип аргумента схемы системы
    if typeSystemShema == 10 then
        SystemShema = xcosDiagramToScilab(SystemShema);//Загружаем схему системы
    elseif typeSystemShema ~= 128 then
        error('The argument SystemShema of function LoadingSystemSchemeWithParamters must be name of xcos file of system''s xcos diagram or xcos diagram type');
    end
    
    //Проверяем аргументы
    setIntegrateParameters = %t;//Необходимость задания параметров интегрирования
    select nArgs
    case 1 then
        return;
    case 2 then
        if (type(SystemParametrsNames) == 1) &&..
           (~isempty(SystemParametrsNames)) then
            absolutTolErr = SystemParametrsNames;
            SystemParametrsNames = [];
            SystemParametrsValues = [];
            relativeTolErr = [];
            simulationTime = [];
            timeStepMax = [];
            Solver = [];
            toleranceTime = [];
        else
            error('The argument absolutTolErr of function SetDiagramParameters must be real scalar');
        end
    case 3 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                return;
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                //Проверяем значения параметров
                if isempty(SystemParametrsValues) then
                    error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
                end
                
                //Задаем необходимость задания параметров интегрирования
                setIntegrateParameters = %f;//Необходимость задания параметров интегрирования
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 4 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                relativeTolErr = [];
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 5 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                Solver = [];
                toleranceTime = [];
            case 10 then
                simulationTime = [];
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 6 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
                toleranceTime = [];
            case 10 then
                timeStepMax = [];
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 7 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                Solver = [];
                toleranceTime = [];
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            select type(SystemParametrsNames)
            case 1 then
                toleranceTime = timeStepMax;
                Solver = simulationTime;
                timeStepMax = relativeTolErr;
                simulationTime = absolutTolErr;
                absolutTolErr = SystemParametrsNames;
                relativeTolErr = SystemParametrsValues;
                SystemParametrsNames = [];
                SystemParametrsValues = [];
            case 10 then
                Solver = [];
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string (or argument absolutTolErr of this function must be matrix of real)');
            end
        end
    case 8 then
        if isempty(SystemParametrsNames) then
            if isempty(SystemParametrsValues) then
                toleranceTime = [];
            else
                error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
            end
        else
            if type(SystemParametrsNames) == 10 then
                toleranceTime = [];
            else
                error('The argument SystemParametrsNames of function SetDiagramParameters must be matrix of string');
            end
        end
    end
    
    //Проверяем корректность типов данных аргументов
    if isempty(SystemParametrsNames) && (~isempty(SystemParametrsValues)) ||..
       (~isempty(SystemParametrsNames)) && isempty(SystemParametrsValues) then
       error('The arguments SystemParametrsNames and SystemParametrsValues of function SetDiagramParameters must be empty together or not empty together');
    end
    if ~isempty(SystemParametrsValues) then
        if (type(SystemParametrsValues) ~= 1) &&..
           (type(SystemParametrsValues) ~= 4) &&..
           (type(SystemParametrsValues) ~= 10) then
           error('The argument SystemParametrsValues of SetDiagramParameters function is array of real, boolean or string of scilab expressions');
        else
           //Приводим заголовки и значения величин к матрице-строке
           SystemParametrsNames = matrix(SystemParametrsNames, 1, -1);
           SystemParametrsValues = matrix(SystemParametrsValues, 1, -1);
           
           //Проверяем соответсвие размеров матриц заголовков и значений величин
           if size(SystemParametrsNames, '*') ~= size(SystemParametrsValues, '*') then
               error('The matrix of names and values of system parametrs dimensions must be equality');
           end
        end
    end
    if setIntegrateParameters then
        TryScalarRealType(absolutTolErr,5);
        TryScalarRealType(relativeTolErr,6);
        TryScalarRealType(simulationTime,7);
        TryScalarRealType(timeStepMax,8);
        TryScalarRealType(Solver,9);
        TryScalarRealType(toleranceTime,10);
    end
    
    //Задаем контекст моделирования
    SystemShema.props.context = SetContext(SystemParametrsNames,...//Имена параметров
                                           SystemParametrsValues,...//Значения параметров
                                           SystemShema.props.context...//Задаваемый контекст
                                           );//Задаем параметры системы
    
    //Задаем параметры моделирования диаграммы
    if setIntegrateParameters then
        if ~isempty(absolutTolErr) then
            SystemShema.props.tol(1) = absolutTolErr;//Задаем абсолютную погрешность
        end
        if ~isempty(relativeTolErr) then
            SystemShema.props.tol(2) = relativeTolErr;//Задаем относительную погрешность
        end
        if ~isempty(simulationTime) then
            SystemShema.props.tf = simulationTime;//Задаем время симуляции
        end
        if ~isempty(timeStepMax) then
            SystemShema.props.tol(7) = timeStepMax;//Задаем vаксимальный шаг интегрирования по времени
        end
        if ~isempty(Solver) then
            SystemShema.props.tol(6) = Solver;//Задаем решатель
        end
        if ~isempty(toleranceTime) then
            SystemShema.props.tol(3) = toleranceTime;//Задаем погрешность для времени
        end
    end
    SystemShema.props.tol(5) = 0.0;//Отключаем реальное время
endfunction 

//Вспомогательные функции
function [context]=SetContext(ParametrsNames,...//Имена параметров
                              ParametrsValues,...//Значения параметров
                              context...//Задаваемый контекст
                              )//задаем контекст
    //Проверяем аргументы
    if isempty(ParametrsNames) || isempty(context) then
        return;
    end
    
    //Отделяем код от исходных данных
    indBorder = find(context == '//end begin parameters values');
    if isempty(indBorder) then
        context0 = [];
    else
        context0 = context((indBorder + 1):$);
        context(indBorder:$) = [];
    end
    
    //Удаляем строки, не являющиеся контекстом
    for ind=1:size(context,'*')//Удаляем комментарии
        indBeginComment = regexp(context(ind),'/\/\//','o');//Индекс начала комментария
        if ~isempty(indBeginComment) then
            context(ind) = part(context(ind),[1:(indBeginComment - 1)]);
        end
    end
    context = strsplit(strcat(context, ';'), ';');//Выделяем присваивания
    context = context(grep(context,'='));//Удаляем строки, не содержащие присваиваний (формируем предварительно контекст)
    
    //Выделяем из контекста имена и значения величин
    context_split = stripblanks(strsplit(strcat(context, ';'), ['=',';']));//Выделяем имена величин и их значения
    context_split = context_split(1:2:size(context_split, '*'));//Получаем имена аргументов в контексте
    
    //Отсеиваем пользовательские величины, не встречающиеся в контексте
    [_,indParametrsNames] = members(ParametrsNames,context_split);
    ParametrsNames(indParametrsNames == 0) = [];//Удаляем имена величин, не входящие в контекст
    ParametrsValues(indParametrsNames == 0) = [];//Удаляем значения величин, не входящих в контекст
    indParametrsNames(indParametrsNames == 0) = [];//Индексы величин, входящих в контекст
    
    //Переводим значения величин в строку
    select type(ParametrsValues)
    case 1 then
        //Переводим числа в строку
        ParametrsValues = string(ParametrsValues);
    case 4 then
        //Переводим логику в строку
        parametrsValues = string(ParametrsValues);
        parametrsValues(ParametrsValues) = '%t';
        parametrsValues(~ParametrsValues) = '%f';
        ParametrsValues = parametrsValues; 
        
        //Чистим память
        clear parametrsValues;
    else
        //Удаляем обрамляющие пробелы
        ParametrsValues = stripblanks(ParametrsValues);
    end
    
    //Удаляем пробелы из имен величин
    ParametrsNames = stripblanks(ParametrsNames);
    
    //Приравниваем именам величин их значения
    parametrs = ParametrsNames + '=' + ParametrsValues;
    
    //Заполняем контекст
    context(indParametrsNames) = parametrs;
    context = [context;context0];
endfunction
function TryScalarRealType(val,numberVal)//Проверяем реальный скалярный тип данных
    if (size(val,'*') > 1) || (type(val) ~= 1) then
       error('The ' + numberVal + 'th argument of ComputingExperiment function are real scalar');
    end
endfunction
