[a, pathlib] = libraryinfo("computing_experimentlib");
clear a;
pathlib = strncpy(pathlib, length(pathlib) - length('macros')) + 'xcos';

//Функция фильтра
function [parametrs]=DynamicFilterFunction1(time, value)
    parametrs=[sum(time.*value) / length(time), sum((time.^2).*value) / (3*length(time))];
endfunction
function [parametr]=DynamicFilterFunction2(time, value)
    parametr=sum(time.*value) / (2*length(time));
endfunction

//Массив функций фильтров
DynamicFiltersFunctions = cell(1, 4);
DynamicFiltersFunctions{1} = DynamicFilterFunction1;
DynamicFiltersFunctions{2} = DynamicFilterFunction2;
DynamicFiltersFunctions{3} = DynamicFilterFunction2;
DynamicFiltersFunctions{4} = DynamicFilterFunction1;

//Параметры диаграммы
SystemParametrsNames = ['b', 'a', 'd'];//Имена параметров
SystemParametrsValues = [ 1.5, 0.81, 3.81;
                         2.25, 0.51, 6.81;
                         1.11, 0.93, 3.51];//Значения параметров

//Гиперпараметры диаграммы
SystemHyperParametrsNames = ['h','e'];//Имена гиперпараметров, определяющих динамику
SystemHyperParametrsValues = [3.69,8.7];//Значения гиперпараметров, определяющих динамику

//Путь к файлу диаграммы
SystemShemaPath = fullfile(pathlib, 'Proba3.zcos');

//Запускаем эталонные примеры
SystemShemae = xcosDiagramToScilab(fullfile(pathlib, 'Proba4e1.zcos'));//Схема системы   
xcos_simulate(SystemShemae, 4);
SystemOutputCharacteristicse = [DynamicFilterFunction1(A3.time,A3.values(:,3)),..
                                DynamicFilterFunction2(A3.time,A3.values(:,2)),..
                                DynamicFilterFunction2(A2.time,A2.values(:,1)),..
                                DynamicFilterFunction1(A1.time,A1.values(:,2))];
SystemShemae = xcosDiagramToScilab(fullfile(pathlib, 'Proba4e2.zcos'));//Схема системы   
xcos_simulate(SystemShemae, 4);
SystemOutputCharacteristicse = [SystemOutputCharacteristicse;...
                                [DynamicFilterFunction1(A3.time,A3.values(:,3)),..
                                 DynamicFilterFunction2(A3.time,A3.values(:,2)),..
                                 DynamicFilterFunction2(A2.time,A2.values(:,1)),..
                                 DynamicFilterFunction1(A1.time,A1.values(:,2))]]; 
SystemShemae = xcosDiagramToScilab(fullfile(pathlib, 'Proba4e3.zcos'));//Схема системы   
xcos_simulate(SystemShemae, 4);
SystemOutputCharacteristicse = [SystemOutputCharacteristicse;...
                                [DynamicFilterFunction1(A3.time,A3.values(:,3)),..
                                 DynamicFilterFunction2(A3.time,A3.values(:,2)),..
                                 DynamicFilterFunction2(A2.time,A2.values(:,1)),..
                                 DynamicFilterFunction1(A1.time,A1.values(:,2))]]; 

//Создаем графические окна
scf(6);
x = 0:0.01:(2*%pi);
plot(x,sin(x));
plot(x,sin(2*x));
scf(7);
plot(x,cos(3*x));

//Проверяем открытые графические окна до тестирования
Wins_begin = winsid();

//Запускаем базовую функцию
[SystemOutputCharacteristics...//Выходные характеристики
 ]=ModelingSystemDynamicEvidence(['A3', 'A3', 'A2', 'A1'],...//Имена выходных характеристик системы
                                 [3, 2, 1, 2],...//Индексы динамик (в именах)
                                 SystemShemaPath,...//Имя схемы системы
                                 DynamicFiltersFunctions,...//Функция сжатия динамик
                                 SystemParametrsNames,...//Имена параметров, определяющих динамику
                                 SystemParametrsValues,...//Значения параметров, определяющих динамику
                                 SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                 SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                 [0.5E-6,0.51E-6,0.57E-6],..//Абсолютная погрешность
                                 [0.69E-6,0.57E-6,0.75E-6],..//Относительная погрешность
                                 [10.0,11.1,15.3],..//Время симуляции
                                 [0.93E-4,0.87E-4,0.81E-4],..//Максимальный шаг интегрирования по времени
                                 [0,3,4]..//Решатель
                                 );

//Сверяем результат расчета с эталонным                        
assert_checkequal(SystemOutputCharacteristics, SystemOutputCharacteristicse);

//Запускаем базовую функцию
[SystemOutputCharacteristics...//Выходные характеристики
 ]=ModelingSystemDynamicEvidence(['A3', 'A3', 'A2', 'A1'],...//Имена выходных характеристик системы
                                 [3, 2, 1, 2],...//Индексы динамик (в именах)
                                 SystemShemaPath,...//Имя схемы системы
                                 ['DynamicFilterFunction1',..
                                  'DynamicFilterFunction2',..
                                  'DynamicFilterFunction2',..
                                  'DynamicFilterFunction1'],...//Функция сжатия динамик
                                 SystemParametrsNames,...//Имена параметров, определяющих динамику
                                 SystemParametrsValues,...//Значения параметров, определяющих динамику
                                 SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                 SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                 [0.5E-6,0.51E-6,0.57E-6],..//Абсолютная погрешность
                                 [0.69E-6,0.57E-6,0.75E-6],..//Относительная погрешность
                                 [10.0,11.1,15.3],..//Время симуляции
                                 [0.93E-4,0.87E-4,0.81E-4],..//Максимальный шаг интегрирования по времени
                                 [0,3,4]..//Решатель
                                 );

//Сверяем результат расчета с эталонным                        
assert_checkequal(SystemOutputCharacteristics, SystemOutputCharacteristicse);

//Загружаем схему системы
SystemShema = xcosDiagramToScilab(SystemShemaPath);

//Запускаем базовую функцию
[SystemOutputCharacteristics...//Выходные характеристики
 ]=ModelingSystemDynamicEvidence(['A3', 'A3', 'A2', 'A1'],...//Имена выходных характеристик системы
                                 [3, 2, 1, 2],...//Индексы динамик (в именах)
                                 SystemShema,...//Имя схемы системы
                                 DynamicFiltersFunctions,...//Функция сжатия динамик
                                 SystemParametrsNames,...//Имена параметров, определяющих динамику
                                 SystemParametrsValues,...//Значения параметров, определяющих динамику
                                 SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                 SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                 [0.5E-6,0.51E-6,0.57E-6],..//Абсолютная погрешность
                                 [0.69E-6,0.57E-6,0.75E-6],..//Относительная погрешность
                                 [10.0,11.1,15.3],..//Время симуляции
                                 [0.93E-4,0.87E-4,0.81E-4],..//Максимальный шаг интегрирования по времени
                                 [0,3,4]..//Решатель
                                 );

//Сверяем результат расчета с эталонным                        
assert_checkequal(SystemOutputCharacteristics, SystemOutputCharacteristicse);

//Запускаем базовую функцию
[SystemOutputCharacteristics...//Выходные характеристики
 ]=ModelingSystemDynamicEvidence(['A3', 'A3', 'A2', 'A1'],...//Имена выходных характеристик системы
                                 [3, 2, 1, 2],...//Индексы динамик (в именах)
                                 SystemShema,...//Имя схемы системы
                                 ['DynamicFilterFunction1',..
                                  'DynamicFilterFunction2',..
                                  'DynamicFilterFunction2',..
                                  'DynamicFilterFunction1'],...//Функция сжатия динамик
                                 SystemParametrsNames,...//Имена параметров, определяющих динамику
                                 SystemParametrsValues,...//Значения параметров, определяющих динамику
                                 SystemHyperParametrsNames,..//Имена гиперпараметров, определяющих динамику
                                 SystemHyperParametrsValues,..//Значения гиперпараметров, определяющих динамику
                                 [0.5E-6,0.51E-6,0.57E-6],..//Абсолютная погрешность
                                 [0.69E-6,0.57E-6,0.75E-6],..//Относительная погрешность
                                 [10.0,11.1,15.3],..//Время симуляции
                                 [0.93E-4,0.87E-4,0.81E-4],..//Максимальный шаг интегрирования по времени
                                 [0,3,4]..//Решатель
                                 );

//Сверяем результат расчета с эталонным                        
assert_checkequal(SystemOutputCharacteristics, SystemOutputCharacteristicse);

//Проверяем оставшиеся графические окна
Wins_end = winsid();//Индексы открытых графических окон
assert_checkequal(Wins_begin,Wins_end);

//Закрываем открытые графические окна
for win=matrix(Wins_end,1,-1)
    delete(scf(win));
end

clear;
